/*
 * Copyright 2001-2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.axis.wsdl.toJava;

import org.apache.axis.wsdl.symbolTable.TypeEntry;

import java.io.IOException;
import java.io.PrintWriter;

/**
 * This is Wsdl2java's Holder Writer.  It writes the <typeName>Holder.java file.
 */
public class JavaHolderWriter extends JavaClassWriter {

    /** Field type */
    private TypeEntry type;

    /**
     * Constructor.
     * 
     * @param emitter 
     * @param type    
     */
    protected JavaHolderWriter(Emitter emitter, TypeEntry type) {

        super(emitter, Utils.holder(type, emitter), "holder");

        this.type = type;
    }    // ctor

    /**
     * Return "public final ".
     * 
     * @return 
     */
    protected String getClassModifiers() {
        return super.getClassModifiers() + "final ";
    }    // getClassModifiers

    /**
     * Return "implements javax.xml.rpc.holders.Holder ".
     * 
     * @return 
     */
    protected String getImplementsText() {
        return "implements javax.xml.rpc.holders.Holder ";
    }    // getImplementsText

    /**
     * Generate the holder for the given complex type.
     * 
     * @param pw 
     * @throws IOException 
     */
    protected void writeFileBody(PrintWriter pw) throws IOException {

        String holderType = type.getName();

        pw.println("    public " + holderType + " value;");
        pw.println();
        pw.println("    public " + className + "() {");
        pw.println("    }");
        pw.println();
        pw.println("    public " + className + "(" + holderType + " value) {");
        pw.println("        this.value = value;");
        pw.println("    }");

        /*** CHANGE FROM ORIGINAL ****/
        pw.println();
        pw.println("    public java.lang.Object getValueAsObject() {");
        pw.println("        return " + JavaBeanWriter.getCastPrimToObj(holderType, "this.value") + ";");
        pw.println("    }");
        pw.println();
        pw.println("    public void setValueAsObject(java.lang.Object val) {");
        pw.println("        this.value = " + JavaBeanWriter.getCastObjToPrim(holderType, "val") + ";");
        pw.println("    }");
        pw.println();
        pw.println("    public java.lang.Class getValueClass() {");
        pw.println("        return " + getClassString(holderType) + ";");
        pw.println("    }");
        pw.println();
        /*** END CHANGE FROM ORIGINAL ****/
    }    // writeOperation

    /*** CHANGE FROM ORIGINAL ****/
	public static String getClassString(String holderType)
	{
        String wrapperClass = holderType;
        if (holderType.indexOf("int") == 0)
        	wrapperClass = "int.class";
        else if (holderType.indexOf("char") == 0)
        	wrapperClass = "char.class";
        else if (holderType.indexOf("short") == 0)
        	wrapperClass = "short.class";
        else if (holderType.indexOf("long") == 0)
        	wrapperClass = "long.class";
        else if (holderType.indexOf("double") == 0)
        	wrapperClass = "double.class";
        else if (holderType.indexOf("float") == 0)
        	wrapperClass = "float.class";
        else if (holderType.indexOf("byte") == 0)
        	wrapperClass = "byte.class";
        else if (holderType.indexOf("boolean") == 0)
        	wrapperClass = "boolean.class";
        else
        	wrapperClass = wrapperClass+".class";
        
        return wrapperClass;
	}
    /*** END CHANGE FROM ORIGINAL ****/
	
    /** Generate a java source file for the holder class.
     * If the emitter works in deploy mode and the class already exists, the source wull not be generated.
     */
    public void generate() throws IOException {
        String fqcn = getPackage() + "." + getClassName();
        if (emitter.isDeploy()) {
            if (!emitter.doesExist(fqcn)) {
                super.generate();
            }
        } else {
            super.generate();
        }
    }
}    // class JavaHolderWriter
